@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2008-2010 Zuza Software Foundation
+# Copyright 2008-2011 Zuza Software Foundation
 #
 # This file is part of Virtaal.
 #
@@ -58,7 +58,6 @@ class SearchMode(BaseMode):
         self.select_first_match = True
         self._search_timeout = 0
         self._unit_modified_id = 0
-        self.unescape = None
 
     def _create_widgets(self):
         # Widgets for search functionality (in first row)
@@ -118,8 +117,10 @@ class SearchMode(BaseMode):
         self._add_widgets()
         self._connect_highlighting()
         self._connect_textboxes()
-        from virtaal.views.markup import unescape
-        self.unescape = unescape
+        unitcont = self.controller.main_controller.unit_controller
+        if self._unit_modified_id:
+            unitcont.disconnect(self._unit_modified_id)
+        self._unit_modified_id = unitcont.connect('unit-modified', self._on_unit_modified)
         if not self.ent_search.get_text():
             self.storecursor.indices = self.storecursor.model.stats['total']
         else:
@@ -153,14 +154,14 @@ class SearchMode(BaseMode):
             textbox.grab_focus()
             buff = textbox.buffer
             buffstr = textbox.get_text()
-            unescaped = self.unescape(buffstr)
 
-            start, end = self._escaped_indexes(unescaped, match.start, match.end)
+            start, end = match.start, match.end
             if hasattr(textbox.elem, 'gui_info'):
-                start = textbox.elem.gui_info.tree_to_gui_index(start)
-                end = textbox.elem.gui_info.tree_to_gui_index(end)
-            start_iter = buff.get_iter_at_offset(start)
-            end_iter = buff.get_iter_at_offset(end)
+                start_iter = textbox.elem.gui_info.treeindex_to_iter(start)
+                end_iter = textbox.elem.gui_info.treeindex_to_iter(end, start_at=(start, start_iter))
+            else:
+                start_iter = buff.get_iter_at_offset(start)
+                end_iter = buff.get_iter_at_offset(end)
 
             buff.select_range(end_iter, start_iter)
             return False
@@ -289,18 +290,6 @@ class SearchMode(BaseMode):
             d[match.unit].append(match)
         return d
 
-    def _escaped_indexes(self, unescaped, start, end):
-        """Returns the indexes of start and end in the escaped version of the
-        given unescaped string."""
-        # Escaping might mean that the indexes should be offset, so we
-        # test to see if escaping comes into play. The unescaped version
-        # will help us calculate how much we need to adjust.
-        leading_segment = unescaped[:end]
-        lines = leading_segment.count(u'\n') + leading_segment.count(u'\t')
-        start = start + lines * 2
-        end = end + lines * 2
-        return (start, end)
-
     def _highlight_matches(self):
         if getattr(self.filter, 're_search', None) is None:
             return
@@ -325,7 +314,6 @@ class SearchMode(BaseMode):
     def _highlight_textbox_matches(self, textbox, select_match=True):
         buff = textbox.buffer
         buffstr = textbox.get_text()
-        unescaped = self.unescape(buffstr)
 
         # Make sure the 'search_highlight' tag in the textbox's tag table
         # is "fresh".
@@ -339,12 +327,24 @@ class SearchMode(BaseMode):
             logging.exception("(Re-)adding search highlighting tag exception:")
 
         select_iters = []
+
+        # We keep the iterator and index pointing to the end of the previous
+        # match so that we continue searching from there for the next match.
+        end_iter = None
+        old_end = -1
+
         for match in self._get_matches_for_textbox(textbox):
-            start, end = self._escaped_indexes(unescaped, match.start, match.end)
+            start, end = match.start, match.end
             if hasattr(textbox.elem, 'gui_info'):
-                start = textbox.elem.gui_info.tree_to_gui_index(start)
-                end   = textbox.elem.gui_info.tree_to_gui_index(end)
-            start_iter, end_iter = buff.get_iter_at_offset(start), buff.get_iter_at_offset(end)
+                if end_iter:
+                    start_iter = textbox.elem.gui_info.treeindex_to_iter(start, start_at=(old_end, end_iter))
+                else:
+                    start_iter = textbox.elem.gui_info.treeindex_to_iter(start)
+                end_iter   = textbox.elem.gui_info.treeindex_to_iter(end, start_at=(start, start_iter))
+                old_end = end
+            else:
+                start_iter, end_iter = buff.get_iter_at_offset(start), buff.get_iter_at_offset(end)
+                old_end = end
             buff.apply_tag_by_name('search_highlight', start_iter, end_iter)
 
             if select_match and textbox.role == 'target' and not select_iters and self.select_first_match:
@@ -398,10 +398,6 @@ class SearchMode(BaseMode):
     def _on_cursor_changed(self, cursor):
         assert cursor is self.storecursor
 
-        unitcont = self.controller.main_controller.unit_controller
-        if self._unit_modified_id:
-            unitcont.disconnect(self._unit_modified_id)
-        self._unit_modified_id = unitcont.connect('unit-modified', self._on_unit_modified)
         self._highlight_matches()
 
     def _on_replace_clicked(self, btn):
