@@ -36,9 +36,15 @@ def run_in_thread(widget, target, args):
     # we make the given widget insensitive to avoid interaction that we can't
     # handle concurrently
     widget.set_sensitive(False)
+    import time
     while thread.isAlive():
         # let gtk process events while target is still running
         gtk.main_iteration(block=False)
+        # Since we are not blocking, we're spinning, which isn't nice. We could
+        # set block=True, but then the window might stay insensitive when the
+        # thread finished, and only exit this loop when it gets another event
+        # (like a mouse move). So we sleep a bit to avoid excessive CPU use.
+        time.sleep(0.03)
 
     widget.set_sensitive(True)
     if q.qsize():
