@@ -25,7 +25,7 @@ import sys
 import logging
 from gtk import gdk
 
-from virtaal.views import recent, theme
+from virtaal.views import theme
 from virtaal.common import pan_app
 
 from baseview import BaseView
@@ -88,6 +88,7 @@ class MainView(BaseView):
             @type  controller: virtaal.controllers.MainController
             @param controller: The controller that this view is "connected" to."""
         self.controller = controller
+        self.modified = False
 
         if os.name == 'nt':
             # Before we do anything else, make sure that stdout and stderr are properly handled.
@@ -124,6 +125,7 @@ class MainView(BaseView):
                 gtk.rc_parse(pan_app.get_abs_data_filename(["themes", "OSX_Leopard_theme", "gtkrc"]))
             except:
                 logging.exception("Couldn't find OSX_Leopard_theme")
+
             # Sometimes we have two resize grips: one from GTK, one from Aqua. We
             # might want to disable the GTK one:
             #self.gui.get_widget('status_bar').set_property("has-resize-grip", False)
@@ -149,10 +151,11 @@ class MainView(BaseView):
                 mnu_prefs = self.gui.get_widget("mnu_prefs")
                 osxapp.insert_app_menu_item(mnu_prefs, 2)
                 self.gui.get_widget("separator_mnu_edit_3").hide()
-                #self.gui.get_widget("menuitem5").hide()
+                gtk.accel_map_load(pan_app.get_abs_data_filename(["virtaal", "virtaal.accel"]))
                 osxapp.ready()
+                osxapp.connect("NSApplicationOpenFile", self._on_osx_openfile_event)
+                osxapp.connect("NSApplicationBlockTermination", self._on_quit)
             except ImportError, e:
-
                 logging.debug("gtk_osxapplication module not found. Expect zero integration with the Mac desktop.")
 
         self.main_window.connect('destroy', self._on_quit)
@@ -184,14 +187,13 @@ class MainView(BaseView):
 
         self.main_window.connect('window-state-event', self._on_window_state_event)
 
-        recent_files = self.gui.get_widget("mnu_recent_files")
-        recent.rc.connect("item-activated", self._on_recent_file_activated)
-        recent_files.set_submenu(recent.rc)
-
         self.controller.connect('controller-registered', self._on_controller_registered)
         self._create_dialogs()
         self._setup_key_bindings()
         self._track_window_state()
+        self._setup_dnd()
+        from gobject import idle_add
+        idle_add(self._setup_recent_files)
         self.main_window.connect('style-set', self._on_style_set)
 
     def _create_dialogs(self):
@@ -203,6 +205,12 @@ class MainView(BaseView):
         self._open_chooser = None
         self._confirm_dialog = None
 
+    def _setup_recent_files(self):
+        from virtaal.views import recent
+        recent_files = self.gui.get_widget("mnu_recent_files")
+        recent.rc.connect("item-activated", self._on_recent_file_activated)
+        recent_files.set_submenu(recent.rc)
+
     @property
     def input_dialog(self):
         # Generic input dialog
@@ -349,6 +357,31 @@ class MainView(BaseView):
             self._window_is_maximized = bool(event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
         self.main_window.connect('window-state-event', on_state_event)
 
+    def _setup_dnd(self):
+        """configures drag and drop"""
+        targets = gtk.target_list_add_uri_targets()
+        # Konqueror needs gtk.gdk.ACTION_MOVE
+        self.main_window.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets, gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
+        self.main_window.connect("drag_data_received", self._on_drag_data_received)
+
+    def _on_drag_data_received(self, w, context, x, y, data, info, time):
+        if sys.platform == 'darwin' or gtk.targets_include_uri(context.targets):
+            # We don't check for valid targets on Mac (darwin) since there is
+            # a bug in target_incude_uri on that platform, no adverse situations
+            # seem to arise but we leave other platforms to do the right thing.
+
+            # the data comes as a string with each URI on a line; lines
+            # terminated with '\r\n. For now we just take the first one:
+            filename = data.data.split("\r\n")[0]
+            if filename.startswith("file://"):
+                # This is a URI, so we handle encoded characters like spaces:
+                import urllib
+                filename = urllib.unquote(filename)
+                #TODO: only bother if the extension is supported?
+                self.controller.open_file(filename)
+
+        return True
+
     def _on_style_set(self, widget, prev_style):
         theme.update_style(widget)
         # on windows the tooltip colour is wrong in inverse themes (bug 1923)
@@ -374,6 +407,11 @@ class MainView(BaseView):
         self.main_window.add_accel_group(accel_group)
 
     def set_saveable(self, value):
+        # Repeatedly doing all of this is unnecessary, and can make the window
+        # title flash slightly. So if the file is already modified, don't
+        # bother redoing all of this.
+        if value and self.modified:
+            return
         menuitem = self.gui.get_widget("mnu_save")
         menuitem.set_sensitive(value)
         menuitem = self.gui.get_widget("mnu_revert")
@@ -524,6 +562,16 @@ class MainView(BaseView):
         self.main_window.show()
         from gobject import threads_init
         threads_init()
+
+        # Uncomment this line to measure startup time until the window shows.
+        # It causes the program to quit immediately when the window is shown:
+        #self.main_window.connect('expose-event', lambda widget, event: gtk.main_quit())
+
+        # Uncomment these lines to measure true startup time. It causes the
+        # program to quit immediately when the last thing added to the gobject
+        # idle queue during startup, is done.
+        #from gobject import idle_add
+        #idle_add(gtk.main_quit)
         gtk.main()
 
     def show_input_dialog(self, title='', message=''):
@@ -554,6 +602,8 @@ class MainView(BaseView):
                 return run_in_thread(self.main_window, native_widgets.kdialog_open_dialog, (self.main_window, title, last_path))
             elif native_widgets.dialog_to_use == 'win32':
                 return run_in_thread(self.main_window, native_widgets.win32_open_dialog, (title, last_path))
+            elif native_widgets.dialog_to_use == 'darwin':
+                return run_in_thread(self.main_window, native_widgets.darwin_open_dialog, (title, last_path))
 
         # otherwise we always fall back to the default code
         if title:
@@ -624,6 +674,8 @@ class MainView(BaseView):
                 return run_in_thread(self.main_window, native_widgets.kdialog_save_dialog, (self.main_window, title, current_filename))
             elif native_widgets.dialog_to_use == 'win32':
                 return run_in_thread(self.main_window, native_widgets.win32_save_dialog, (title, current_filename))
+            elif native_widgets.dialog_to_use == 'darwin':
+                return run_in_thread(self.main_window, native_widgets.darwin_save_dialog, (title, current_filename))
 
         # otherwise we always fall back to the default code
         if title:
@@ -782,6 +834,7 @@ class MainView(BaseView):
             self.gui.get_widget('mnu_binary_export').set_sensitive(True)
 
         self.status_bar.set_sensitive(True)
+        from virtaal.views import recent
         if store_controller.project:
             if not store_controller._archivetemp:
                 recent.rm.add_item('file://' + store_controller.get_bundle_filename())
@@ -789,7 +842,11 @@ class MainView(BaseView):
             if getattr(self, '_uri', None):
                 recent.rm.add_item(self._uri)
             else:
-                recent.rm.add_item('file://' + os.path.abspath(store_controller.store.filename))
+                if os.name == 'nt':
+                    url = 'file:///' + os.path.abspath(store_controller.store.filename)
+                else:
+                    url = 'file://' + os.path.abspath(store_controller.store.filename)
+                recent.rm.add_item(url)
 
     def _on_window_state_event(self, widget, event):
         mnu_fullscreen = self.gui.get_widget('mnu_fullscreen')
@@ -797,3 +854,16 @@ class MainView(BaseView):
 
     def _on_app_pressed(self, btn):
         self.app_menu.popup(None, None, None, 0, 0)
+
+    def _on_osx_openfile_event(self, macapp, filename):
+        # Note! A limitation of the current GTK-OSX code
+        # (2.18) is that we cannot perform any operations
+        # involving the GTK run-loop within this handler,
+        # therefore we schedule the load to occur afterwards.
+        # See gdk/quartz/gdkeventloop-quartz.c in the GTK+ source. 
+        from gobject import idle_add
+        def callback():
+            self.controller.open_file(filename)
+        idle_add(callback) 
+        # We must indicate we handled this or crash
+        return True 
